@@ -135,6 +135,7 @@ static MicrofeedError* update_feed(MicrofeedFeed* feed, int user_initiated, void
 		error = microfeed_error_new(MICROFEED_ERROR_CONNECTION_FAILED, "Could not get the feed.");
 	}
 
+	microfeed_http_free(http);
 	free(url);
 
 	return error;
@@ -145,6 +146,7 @@ static MicrofeedError* modify_item(MicrofeedFeed *feed, MicrofeedItem *existing_
 	char* encoded_text;
 	const char* referred;
 	char* post_data;
+	const char* source;
 	char* site;
 	char* url;
 	MicrofeedHttp* http;
@@ -166,6 +168,9 @@ static MicrofeedError* modify_item(MicrofeedFeed *feed, MicrofeedItem *existing_
 		post_data = microfeed_util_string_concatenate("status=", encoded_text, NULL);		
 	}
 	free(encoded_text);
+	if ((source = microfeed_item_get_property(new_item, ".twitter.source"))) {
+		post_data = microfeed_util_string_append(post_data, "&source=", source, NULL);
+	}
 
 	site = microfeed_publisher_get_setting_value(microfeed_feed_get_publisher(feed), "service.url", "http://twitter.com");
 	url = microfeed_util_string_concatenate(site, "/statuses/update.json", NULL);
@@ -178,6 +183,7 @@ static MicrofeedError* modify_item(MicrofeedFeed *feed, MicrofeedItem *existing_
 		return microfeed_error_new(MICROFEED_ERROR_COMMUNICATION_FAILED, "Invalid reply when updating status.");
 	}
 	
+	microfeed_http_free(http);
 	microfeed_json_free(json);
 	free(url);
 	free(post_data);
@@ -206,6 +212,7 @@ static MicrofeedError* mark_item(MicrofeedFeed* feed, const char* uid, int mark_
 		return microfeed_error_new(MICROFEED_ERROR_COMMUNICATION_FAILED, "Invalid reply when changing favorite status.");
 	}
 	
+	microfeed_http_free(http);
 	microfeed_json_free(json);
 	free(url);
 	
