@@ -25,36 +25,43 @@
 #include <microfeed-common/microfeedprotocol.h>
 #include <microfeed-common/microfeedmisc.h>
 #include <microfeed-provider/microfeedhttp.h>
+#include <microfeed-common/microfeedobject.h>
 
 #include "twitter.h"
 
 #include <string.h>
 #include <stdio.h>
 
+static MicrofeedPublisher* instantiate_publisher(MicrofeedProvider* provider, const char* publisher_identifier, const char* directory, void* user_data);
+static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data);
 static void no_more_publishers(MicrofeedProvider* publisher, void* user_data);
-static MicrofeedFeed* create_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data);
 static void initialize_settings(MicrofeedPublisher* publisher, void* user_data);
 static int update_setting(MicrofeedPublisher* publisher, const char* key, const char* value, void* user_data);
-static MicrofeedError* download_image(MicrofeedPublisher* publisher, const char* uid, void** data, size_t* length, void* user_data);
-static void* add_timeout(MicrofeedPublisher* publisher, unsigned long int interval, MicrofeedPublisherTimeoutHandler handler, void* data, void* user_data);
-static void remove_timeout(MicrofeedPublisher* publisher, void* timeout_implementation, void* user_data);
+static void* add_timeout(MicrofeedProvider* provider, unsigned long int interval, MicrofeedProviderTimeoutHandler handler, void* data, void* user_data);
+static void remove_timeout(MicrofeedProvider* provider, void* timeout_implementation, void* user_data);
+static MicrofeedError* download_image(MicrofeedFeed* feed, const char* uid, void** data, size_t* length, void* user_data);
 
-MicrofeedProviderCallbacks provider_callbacks = {
-	no_more_publishers,
-	{
-		create_feed,
-		initialize_settings,
-		update_setting,
-		download_image,
-		NULL,
-		add_timeout,
-		remove_timeout
-	}
+static MicrofeedProviderCallbacks provider_callbacks = {
+	.instantiate_publisher = instantiate_publisher,
+	.no_more_publishers = no_more_publishers,
+	.add_timeout = add_timeout,
+	.remove_timeout = remove_timeout
+};
+
+static MicrofeedPublisherCallbacks publisher_callbacks = {
+	.instantiate_feed = instantiate_feed,
+	.initialize_settings = initialize_settings,
+	.update_setting = update_setting,
+};
+
+static MicrofeedFeedCallbacks images_feed_callbacks = {
+	.download_item_data = download_image
 };
 
 int main(void) {
 	MicrofeedMain* microfeed_main;
 	MicrofeedProvider* provider;
+	MicrofeedWeakReference* weak_reference;
 	
 	//dbus_connection_set_change_sigpipe(FALSE);
 	dbus_threads_init_default();
@@ -63,10 +70,16 @@ int main(void) {
 
 	microfeed_main = microfeed_main_new();
 	provider = microfeed_provider_new(PROVIDER_ID_TWITTER, microfeed_main_get_dbus_connection(microfeed_main), &provider_callbacks, microfeed_main);
+	weak_reference = microfeed_object_get_weak_reference(provider, MicrofeedProvider);
 
-	microfeed_main_loop(microfeed_main);
+	do {
+		microfeed_main_loop(microfeed_main);
+		
+		microfeed_object_unref(provider, MicrofeedProvider);
+		provider = microfeed_weak_reference_get_object(weak_reference, MicrofeedProvider);
+	} while (provider);
 
-	microfeed_provider_free(provider);
+	microfeed_weak_reference_unref(weak_reference);
 
 	return 0;
 }
@@ -75,25 +88,58 @@ static void no_more_publishers(MicrofeedProvider* provider, void* user_data) {
 	MicrofeedMain* microfeed_main;
 	
 	microfeed_main = (MicrofeedMain*)user_data;
+	
 	microfeed_main_exit(microfeed_main);
 }
 
-static MicrofeedFeed* create_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data) {
+static MicrofeedPublisher* instantiate_publisher(MicrofeedProvider* provider, const char* publisher_identifier, const char* directory, void* user_data) {
+	MicrofeedConfiguration* configuration = NULL;
+	MicrofeedPublisher* publisher;
+	
+	if (!directory) {
+		configuration = microfeed_configuration_new();
+		directory = microfeed_configuration_get_default_publisher_directory(configuration);
+	}
+	
+	publisher = microfeed_publisher_new(provider, publisher_identifier, directory, &publisher_callbacks, user_data);
+
+	if (configuration) {
+		microfeed_configuration_free(configuration);
+	}
+
+	return publisher;
+}
+
+static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data) {
 	MicrofeedFeed* feed = NULL;
 	MicrofeedMain* microfeed_main;
 	size_t length;
 	char* url;
+	MicrofeedFeed* images_feed;
 	
 	microfeed_main = (MicrofeedMain*)user_data;
 
 	if (!strcmp(uri, MICROFEED_FEED_URI_OVERVIEW)) {
 		url = strdup("/statuses/friends_timeline.json");
 		feed = microfeed_feed_new(publisher, uri, "Overview", MICROFEED_FEED_PERMISSION_ADD, &overview_feed_callbacks, url);
+		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
+		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
+		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_IMAGE, images_feed);
+		microfeed_object_unref(images_feed, MicrofeedFeed);
 	} else if (!strcmp(uri, MICROFEED_FEED_URI_CONTACTS)) {
 		feed = microfeed_feed_new(publisher, uri, "Contacts", MICROFEED_FEED_PERMISSION_ADD, &contacts_callbacks, NULL);
+		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
+		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
+		microfeed_object_unref(images_feed, MicrofeedFeed);
+	} else if (!strcmp(uri, MICROFEED_FEED_URI_IMAGES)) {
+		feed = microfeed_feed_new(publisher, uri, "Images", MICROFEED_FEED_PERMISSION_NONE, &images_feed_callbacks, NULL);
 	} else if ((length = microfeed_util_string_starts_with(uri, MICROFEED_FEED_URI_USER_PREFIX))) {
 		url = microfeed_util_string_concatenate("/statuses/user_timeline.json?user_id=", uri + strlen(MICROFEED_FEED_URI_USER_PREFIX), NULL);
 		feed = microfeed_feed_new(publisher, uri, uri + length, MICROFEED_FEED_PERMISSION_NONE, &feed_callbacks, url);
+		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
+		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
+		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_IMAGE, images_feed);
+		microfeed_object_unref(images_feed, MicrofeedFeed);
 	}
 
 	return feed;	
@@ -109,7 +155,7 @@ static int update_setting(MicrofeedPublisher* publisher, const char* key, const
 	return 1;
 }
 
-static MicrofeedError* download_image(MicrofeedPublisher* publisher, const char* uid, void** data, size_t* length, void* user_data) {
+static MicrofeedError* download_image(MicrofeedFeed* feed, const char* uid, void** data, size_t* length, void* user_data) {
 	char* buffer;
 	MicrofeedError* error = NULL;
 	MicrofeedHttp* http;
@@ -128,7 +174,7 @@ static MicrofeedError* download_image(MicrofeedPublisher* publisher, const char*
 
 typedef struct {
 	unsigned long int interval;
-	MicrofeedPublisherTimeoutHandler handler;
+	MicrofeedProviderTimeoutHandler handler;
 	void* data;
 	MicrofeedTimeout* timeout;
 } TimeoutData;
@@ -144,7 +190,7 @@ static void timeout_callback(MicrofeedMain* microfeed_main, void* user_data) {
 	}
 }
 
-static void* add_timeout(MicrofeedPublisher* publisher, unsigned long int interval, MicrofeedPublisherTimeoutHandler handler, void* data, void* user_data) {
+static void* add_timeout(MicrofeedProvider* provider, unsigned long int interval, MicrofeedProviderTimeoutHandler handler, void* data, void* user_data) {
 	MicrofeedMain* microfeed_main;
 	TimeoutData* timeout_data;
 
@@ -158,7 +204,7 @@ static void* add_timeout(MicrofeedPublisher* publisher, unsigned long int interv
 	return timeout_data;
 }
 
-static void remove_timeout(MicrofeedPublisher* publisher, void* timeout_implementation, void* user_data) {
+static void remove_timeout(MicrofeedProvider* provider, void* timeout_implementation, void* user_data) {
 	MicrofeedMain* microfeed_main;
 	TimeoutData* timeout_data;
 	
